/////////////////////////////////////////////////////////////////////////////////
//                                                                             //
// <copyright company="Objectis">                                              //
//                                                                             //
// Copyright (c) 2013, All Right Reserved, http://www.objectis.ch/             //
//                                                                             //
// This source is subject to the Objectis Permissive License.                  //
// Please see the License contract for more information.                       //      
// All other rights reserved.                                                  //
//                                                                             //
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY      //
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE         //
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A                  //
// PARTICULAR PURPOSE.                                                         //
//                                                                             //
// </copyright>                                                                //
//                                                                             //
/////////////////////////////////////////////////////////////////////////////////

#ifndef __OCFE_H_
#define __OCFE_H_

#include "OcfEmbeddedPort.h"
#include "OcfEmbeddedProtocol.h"

typedef enum
{
	OCFE_TYPE_BOOL = 1,
	OCFE_TYPE_CHAR = 2,
	OCFE_TYPE_SHORT = 3,
	OCFE_TYPE_INT = 4,
	OCFE_TYPE_LONG = 5

#ifdef OCFE_ENABLE_FLOAT
	,OCFE_TYPE_FLOAT = 6
#endif

#ifdef OCFE_ENABLE_DOUBLE
	,OCFE_TYPE_DOUBLE = 7
#endif

#ifdef OCFE_ENABLE_STRING
	,OCFE_TYPE_STRING = 8
#endif

#ifdef OCFE_ENABLE_FUNCTION
	,OCFE_TYPE_FUNCTION_VOID = 9
	,OCFE_TYPE_FUNCTION_RET = 10
#endif

} OcfeType;

typedef short OcfeInt16;
typedef unsigned short OcfeUInt16;
typedef char OcfeChar8;
typedef unsigned char OcfeUInt8;
#ifdef OCFE_ENABLE_FUNCTION
	typedef	void (*ptrFonctionVoid)(void);
	typedef char*(*pftFonctionRet)(void);
#endif
typedef char OcfeBuffer[OCFE_COMMUNICATION_BUFFER_SIZE];

typedef struct
{
	OcfeUInt16 PosEndString;
	OcfeUInt16 CheckSum;
	OcfeBuffer Buffer;
	OcfeUInt8 ElementPositions[OCFE_MAX_FRAME_ELEMENT_COUNT];
} OcfeContext;

typedef OcfeContext OcfePacketContext;

typedef struct
{
	OcfeUInt16 CharToWrite;
	OcfeUInt16 Index;
	OcfeContext context;
} OcfeStreamContext;



// input : buffer contains zero terminated input text
// Returns the number of characters placed in buffer

#ifdef __cplusplus
extern "C"
{
#endif

OcfeUInt16 OcfeExecute(OcfeVariableEntry ocfeDictionnary[], OcfeContext *context, OcfeUInt16 inputDataLength);
OcfeUInt16 OcfeCharacterReceived(OcfeVariableEntry* ocfeDictionnary, OcfeStreamContext *context, char c);
OcfeUInt16 OcfeCharacterToSend(OcfeStreamContext *context, char* c);

OcfeUInt16 OcfeFletcher16Checksum(const char *dataChar, OcfeUInt16 count);


#ifdef __cplusplus
}
#endif

#endif
