/////////////////////////////////////////////////////////////////////////////////
//                                                                             //
// <copyright company="Objectis">                                              //
//                                                                             //
// Copyright (c) 2013, All Right Reserved, http://www.objectis.ch/             //
//                                                                             //
// This source is subject to the Objectis Permissive License.                  //
// Please see the License contract for more information.                       //      
// All other rights reserved.                                                  //
//                                                                             //
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY      //
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE         //
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A                  //
// PARTICULAR PURPOSE.                                                         //
//                                                                             //
// </copyright>                                                                //
//                                                                             //
/////////////////////////////////////////////////////////////////////////////////

#ifndef __OCFE_PORT_H_
#define __OCFE_PORT_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//Concurent access protection
#define LOCK				//Interrupt disable
#define UNLOCK				//Interrupt enable

//Memory area for Ocf Embedded (if necessary)
#define OCFE_MEMORY_AREA

// Sizes and precisions wanted
#define OCFE_FLOAT_PRECISION 3
#define OCFE_DOUBLE_PRECISION 6

#define OCFE_MAX_VAR_NAME_LENGTH		20
#define OCFE_MAX_VALUE_LENGTH			10

// Comment unwanted features :
#define OCFE_ENABLE_BOOL
#define OCFE_ENABLE_INT
#define OCFE_ENABLE_LONG
//#define OCFE_ENABLE_FLOAT
//#define OCFE_ENABLE_DOUBLE
#define OCFE_ENABLE_STRING
#define OCFE_ENABLE_FUNCTION

#define OCFE_ENABLE_FRAME_NUMERATOR
#define OCFE_ENABLE_CHECKSUM

// Utils functions
#define OCFE_STRING_COPY(dest, src)										strcpy((char*)dest, (char*)src)
#define OCFE_STRING_LENGTH(string)										strlen(string)
#define OCFE_STRING_EQUAL(string1, string2)								(strcmp(string1, string2) == 0)

#define OCFE_VAL_TO_STRING_PRECISION(string, val, format, precision)    sprintf(string, "%.*"#format, precision, val)
#define OCFE_VAL_TO_STRING(string, val, format)							sprintf(string, "%"#format, val)

#define OCFE_BOOL_FROM_STRING(val, string)								if(OCFE_STRING_EQUAL(string, "False") || *string == '0') val = 0; else val = 1
#define OCFE_BOOL_TO_STRING(string, val)								OCFE_STRING_COPY(string, (val == 1 ? "True" : "False"))
#define OCFE_CHAR_FROM_STRING(val, string)								(val = (char)atoi(string))
#define OCFE_CHAR_TO_STRING(string, val)								OCFE_VAL_TO_STRING(string, (char)val, d)
#define OCFE_SHORT_FROM_STRING(val, string)								(val = (short)atoi(string))
#define OCFE_SHORT_TO_STRING(string, val)								OCFE_VAL_TO_STRING(string, (short)val, d)
#define OCFE_INT_FROM_STRING(val, string)								(val = atoi(string))
#define OCFE_INT_TO_STRING(string, val)									OCFE_VAL_TO_STRING( string, val, d)
#define OCFE_LONG_FROM_STRING(val, string)								(val = atol(string))
#define OCFE_LONG_TO_STRING(string, val)								OCFE_VAL_TO_STRING(string, val, ld)
#define OCFE_FLOAT_FROM_STRING(val, string)								(val = (float)atof(string))
#define OCFE_FLOAT_TO_STRING(string, val)								OCFE_VAL_TO_STRING_PRECISION(string, val, f, OCFE_FLOAT_PRECISION)
#define OCFE_DOUBLE_FROM_STRING(val, string)							(val = atof(string))
#define OCFE_DOUBLE_TO_STRING(string, val)								OCFE_VAL_TO_STRING_PRECISION(string, val, Lf, OCFE_DOUBLE_PRECISION)

#endif
