/////////////////////////////////////////////////////////////////////////////////
//                                                                             //
// <copyright company="Objectis">                                              //
//                                                                             //
// Copyright (c) 2013, All Right Reserved, http://www.objectis.ch/             //
//                                                                             //
// This source is subject to the Objectis Permissive License.                  //
// Please see the License contract for more information.                       //      
// All other rights reserved.                                                  //
//                                                                             //
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY      //
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE         //
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A                  //
// PARTICULAR PURPOSE.                                                         //
//                                                                             //
// </copyright>                                                                //
//                                                                             //
/////////////////////////////////////////////////////////////////////////////////

#include "OcfEmbedded.h"
#include "OcfEmbeddedPort.h"
#include <winsock2.h>

//Required to tell the linker we need winsock 2
#pragma comment(lib, "Ws2_32.lib")

#define PORT 10000

struct{
	int A;
	long B;
}MyStruct;

int C;

void IncrementA()
{
	printf("IncrementA() called\n");
	MyStruct.A++;
}

OCFE_BEGIN(ocfEntries)
	OCFE_VARIABLE(MyStruct.A, OCFE_TYPE_INT)
	OCFE_VARIABLE(MyStruct.B, OCFE_TYPE_LONG)
	OCFE_VARIABLE(C, OCFE_TYPE_INT)
	OCFE_FUNCTION_VOID(IncrementA)
OCFE_END

int main(int argc, char* argv[])
{
	//Structures needed for the creation of our socket
	SOCKET listener;
	SOCKET sender;

	//Windows-specific variable used for starting up a socket
	WSADATA wsaData;

	//local is address of the server (this program)
	struct sockaddr_in localAddr;
	//source is address of the client (usually oStudio or your own application)
	struct sockaddr_in sourceAddr;
	//The length of the source address, used in recvfrom
	int sourceAddrLen;

	//Used as the result of the WSAStartup function
	int startupResult;

	//The packet context is used to store the data received by UDP
	OcfePacketContext packetContext;
	//The amount of bytes received
	int bytesReceived = 0;

	//The amount of bytes ready to be sent
	int bytesToSend = 0;

	//We set the address type to be AF_INET (IPv4)
	localAddr.sin_family = AF_INET;
	//We bind the socket to INADDR_ANY (0.0.0.0, any address on this machine)
	localAddr.sin_addr.s_addr = INADDR_ANY;
	//The port needs to be passed through htons (network byte order)
	localAddr.sin_port = htons(PORT);

	//First and foremost, we need to startup winsocks
	if ( (startupResult = WSAStartup(0x202, &wsaData)) != 0 )
	{
		printf("Error starting up winsock. Error #%d\n",startupResult);
		return EXIT_FAILURE;
	}

	//Now, we can open the socket
	//SOCK_DGRAM means we are using the Datagram protocol (UDP)
	listener = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if ( listener == INVALID_SOCKET )
	{
		printf("Error creating socket. Error #%d\n",WSAGetLastError());
		return EXIT_FAILURE;
	}

	//Now we bind our socket to our local address
	if ( bind(listener, (struct sockaddr*)&localAddr, sizeof(localAddr)) )
	{
		printf("Error binding to socket. Error #%d\n",WSAGetLastError());
		return EXIT_FAILURE;
	}

	sender = listener;

	sourceAddrLen = sizeof(sourceAddr);

	printf("  ____  _     _           _   _                  __ _                          \n");
	printf(" / __ \\| |   (_)         | | (_)                / _| |                         \n");
	printf("| |  | | |__  _  ___  ___| |_ _ ___   ___  ___ | |_| |___      ____ _ _ __ ___ \n");
	printf("| |  | | '_ \\| |/ _ \\/ __| __| / __| / __|/ _ \\|  _| __\\ \\ /\\ / / _` | '__/ _ \\\n");
	printf("| |__| | |_) | |  __/ (__| |_| \\__ \\ \\__ \\ (_) | | | |_ \\ \V  \V / (_| | | |  __/\n");
	printf(" \\____/|_.__/| |\\___|\\___|\\__|_|___/ |___/\\___/|_|  \\__| \\_/\\_/ \\__,_|_|  \\___|\n");
	printf("            _/ |                                                               \n");
	printf("           |__/                                                                 \n");

	printf("This is a Ocf Server (OcfEmbedded) demo. All rights reserved.\n");
	printf("Please contact Objectis S.A. on www.objectis-software.com for more information.\n");
	printf("Server started on port : %d\n\n", PORT);

	//We are ready to receive messages!
	while(1)
	{
		//We call recvfrom which is the UDP way of receiving a packet. This function
		//is blocking. We will get all the information we need about the client in 
		//the sourceAddr structure
		bytesReceived = recvfrom(sender, 
			packetContext.Buffer, 
			sizeof(OcfeBuffer), 
			0, 
			(struct sockaddr*)&sourceAddr, 
			&sourceAddrLen);

		if ( bytesReceived == SOCKET_ERROR )
		{
			printf("Error receiving data. Error #%d\n",WSAGetLastError());
			return EXIT_FAILURE;
		}

		//We are ready to send this text to OcfEmbedded for parsing
		//Here is where you should put your call to OcfeExecute
		bytesToSend = OcfeExecute(ocfEntries, &packetContext, bytesReceived);

		//Send the response
		if ( sendto(sender, 
					packetContext.Buffer, 
					bytesToSend, 
					0, 
					(struct sockaddr*)&sourceAddr, 
					sizeof(sourceAddr)) 
						== SOCKET_ERROR )
		{
			printf("Error sending data. Error #%d\n",WSAGetLastError());
			return EXIT_FAILURE;
		}
	}

	return 0;
}
